@@ -9,7 +9,7 @@
 # Website: continualai.org                                                     #
 ################################################################################
 
-""" Endless-CL-Sim Dataset """
+"""Endless-CL-Sim Dataset."""
 
 from pathlib import Path
 import glob
@@ -25,33 +25,39 @@ from PIL import Image
 from torch.utils.data import Dataset
 
 from avalanche.benchmarks.datasets import default_dataset_location
-from avalanche.benchmarks.datasets.endless_cl_sim import \
-    endless_cl_sim_data
-from avalanche.benchmarks.datasets.downloadable_dataset import \
-    DownloadableDataset
+from avalanche.benchmarks.datasets.endless_cl_sim import endless_cl_sim_data
+from avalanche.benchmarks.datasets.downloadable_dataset import (
+    DownloadableDataset,
+)
 
 
 class ClassificationSubSequence(Dataset):
     """Image-Patch Classification Subsequence Dataset"""
 
-    def __init__(self, file_paths, targets, patch_size=64,
-                 labelmap_path=None, transform=None, target_transform=None):
-        """
-        Dataset containing image-patches and targets for one subsequence of
+    def __init__(
+        self,
+        file_paths,
+        targets,
+        patch_size=64,
+        labelmap_path=None,
+        transform=None,
+        target_transform=None,
+    ):
+        """Dataset containing image-patches and targets for one subsequence of
         an endless continual learning simulator's sequence, that has been
         converted for image-patch classification.
 
         :param file_paths: List that contains the paths to all images files
             that are part of this subsequence.
-        :param targets: List that contains the targets (`object category 
+        :param targets: List that contains the targets (`object category
             names` (str)) for each respective image.
-        :param patch_size: Int defining the quadratic patch-size the 
+        :param patch_size: Int defining the quadratic patch-size the
             image-patches are resized to.
         :param labelmap_path: Path to a `labelmap.json` file that specifies
-            a mapping from `object category names` to labels. 
-        :param transform: Eventual transformations to be applied to the image 
+            a mapping from `object category names` to labels.
+        :param transform: Eventual transformations to be applied to the image
             data.
-        :param target_transform: Eventual transformations to be applied to the 
+        :param target_transform: Eventual transformations to be applied to the
             target data.
         """
         self.file_paths = file_paths
@@ -66,8 +72,11 @@ class ClassificationSubSequence(Dataset):
 
     def _pil_loader(self, file_path):
         with open(file_path, "rb") as f:
-            img = Image.open(f).convert("RGB").resize(
-                (self.patch_size, self.patch_size), Image.NEAREST)
+            img = (
+                Image.open(f)
+                .convert("RGB")
+                .resize((self.patch_size, self.patch_size), Image.NEAREST)
+            )
         return img
 
     def _load_labelmap(self, path):
@@ -107,30 +116,36 @@ class ClassificationSubSequence(Dataset):
 class VideoSubSequence(Dataset):
     """Video Subsequence Dataset"""
 
-    def __init__(self, file_paths, target_paths, 
-                 segmentation_file, classmap_file=None, patch_size=(240, 135), 
-                 transform=None, target_transform=None):
-        """
-        Dataset that contains the (image) data and semantic segmentation targets 
-        for one subsequence of a video sequence. 
+    def __init__(
+        self,
+        file_paths,
+        target_paths,
+        segmentation_file,
+        classmap_file=None,
+        patch_size=(240, 135),
+        transform=None,
+        target_transform=None,
+    ):
+        """Dataset that contains the (image) data and semantic segmentation
+        targets for one subsequence of a video sequence.
 
-        :param file_paths: List containing the paths to all images files that 
+        :param file_paths: List containing the paths to all images files that
             are part of this subsequence.
-        :param target_paths: List containing the paths to all target files 
-            corresponding to the `file_paths`. 
-        :param segmentation_file: Path to a `segmentation.json` file that 
-            specifies a mapping from label indices to object  
-            (or object category) names. Defaults to None, which loads a 
+        :param target_paths: List containing the paths to all target files
+            corresponding to the `file_paths`.
+        :param segmentation_file: Path to a `segmentation.json` file that
+            specifies a mapping from label indices to object
+            (or object category) names. Defaults to None, which loads a
             predefined default mapping.
         :param classmap_file: Path to a `classmap.json' file that specifies
-            the mapping from object (or object category) names to a 
+            the mapping from object (or object category) names to a
             respective label. Defaults to None, which loads a predefined
             default mapping.
         :param patch_size: Size of the images and target data to be resized to.
             Defaults to (240, 135).
-        :param transform: Eventual transformations to be applied to the image 
+        :param transform: Eventual transformations to be applied to the image
             data.
-        :param target_transform: Eventual transformations to be applied to the 
+        :param target_transform: Eventual transformations to be applied to the
             target data.
         """
         self.file_paths = file_paths
@@ -146,7 +161,8 @@ class VideoSubSequence(Dataset):
 
         # Init labelmap
         self.labelmap = self._load_labelmap(
-            labelmap_file=self.segmentation_file)
+            labelmap_file=self.segmentation_file
+        )
         return
 
     def _pil_loader(self, file_path, is_target=False):
@@ -154,8 +170,11 @@ class VideoSubSequence(Dataset):
             convert_identifier = "RGB"
             if is_target:
                 convert_identifier = "L"
-            img = Image.open(f).convert(convert_identifier).resize(
-                (self.patch_size[0], self.patch_size[1]), Image.NEAREST)
+            img = (
+                Image.open(f)
+                .convert(convert_identifier)
+                .resize((self.patch_size[0], self.patch_size[1]), Image.NEAREST)
+            )
         return img
 
     def _load_classmap(self, classmap_file):
@@ -197,9 +216,9 @@ class VideoSubSequence(Dataset):
         raise ValueError(f"label: {label} could not be converted!")
 
     def _convert_target(self, target):
-        """ 
-        Converts segmentation target (instance-segmented) according to classmap
-        """ 
+        """Converts segmentation target (instance-segmented) according to
+        classmap.
+        """
         # Get all unique labels in target
         target = target.copy()
         unique_labels = torch.unique(torch.tensor(target)).numpy()
@@ -231,71 +250,78 @@ class VideoSubSequence(Dataset):
 
 
 class EndlessCLSimDataset(DownloadableDataset):
-    """ Endless Continual Leanring Simulator Dataset """ 
+    """Endless Continual Leanring Simulator Dataset"""
 
     def __init__(
-            self,
-            root: Union[str, Path] = None,
-            *,
-            scenario=None, patch_size=64,
-            transform=None, target_transform=None,
-            download=True, semseg=False,
-            labelmap_path=None):
-        """
-        Creates an instance of the Endless-Continual-Leanring-Simulator Dataset.
-        This dataset is able to download and prepare datasets derived from the 
-        Endless-Continual-Learning Simulator, including settings of incremental 
-        classes, decrasing illumination, and shifting weather conditions, as 
-        described in the paper `A Procedural World Generation Framework for 
-        Systematic Evaluation of Continual Learning' 
-        (https://arxiv.org/abs/2106.02585). Also custom datasets are supported 
-        when following the same structure. Such can be obtained from the 
-        Endless-CL-Simulator standalone application 
-        (https://zenodo.org/record/4899294).
+        self,
+        root: Union[str, Path] = None,
+        *,
+        scenario=None,
+        patch_size=64,
+        transform=None,
+        target_transform=None,
+        download=True,
+        semseg=False,
+        labelmap_path=None,
+    ):
+        """Creates an instance of the Endless-Continual-Leanring-Simulator
+        Dataset.
+
+        This dataset is able to download and prepare datasets derived from the
+        Endless-Continual-Learning Simulator, including settings of incremental
+        classes, decrasing illumination, and shifting weather conditions, as
+        described in the paper `A Procedural World Generation Framework for
+        Systematic Evaluation of Continual Learning 
+        <https://arxiv.org/abs/2106.02585>`__.
+        Also custom datasets are supported
+        when following the same structure. Such can be obtained from the
+        `Endless-CL-Simulator standalone application
+        <https://zenodo.org/record/4899294>`__.
 
         Please note:
-        1) The EndlessCLSimDataset does not provide examples directly, but 
-        SubsequenceDatasets (ClassificationSubSequence, VideoSubSequence). Each 
-        SubSequenceDataset will contain the samples for one respective sub 
-        sequence. 
+        1) The EndlessCLSimDataset does not provide examples directly, but
+        SubsequenceDatasets (ClassificationSubSequence, VideoSubSequence). Each
+        SubSequenceDataset will contain the samples for one respective sub
+        sequence.
 
-        2) For video sequences currently only one sequence per dataset is 
+        2) For video sequences currently only one sequence per dataset is
         supported!
 
         :param root: root for the datasets data. Defaults to None, which means
-        that the default location for 'endless-cl-sim' will be used.
-        :param scenario: identifier for the dataset to be used. 
-        Predefined options are 'Classes', for incremental classes scenario, 
-            'Illumination', for the decreasing lighting scenario, 
+            that the default location for 'endless-cl-sim' will be used.
+        :param scenario: identifier for the dataset to be used.
+            Predefined options are 'Classes', for incremental classes scenario,
+            'Illumination', for the decreasing lighting scenario,
             and 'Weather', for the scenario of shifting weather conditions.
-            To load a custom (non-predefined/downloadable) dataset, the 
+            To load a custom (non-predefined/downloadable) dataset, the
             identifier needs to be set to None. Defaults to None.
-        :param patch_size: optional size of image data to be loaded. 
+        :param patch_size: optional size of image data to be loaded.
             For classification the patch_size is of type `int`, because we only
-            consider quadratic input sizes. If the `semseg` flag is set, 
+            consider quadratic input sizes. If the `semseg` flag is set,
             the patch_size type is `tuple`, with `(width, height)`.
-        :param transform: optional transformations to be applied to the image 
+        :param transform: optional transformations to be applied to the image
             data.
-        :param target_transform: optional transformations to be applied to the 
+        :param target_transform: optional transformations to be applied to the
             targets.
-        :param download: boolean to automatically download data. 
+        :param download: boolean to automatically download data.
             Defaults to True.
-        :param semseg: boolean to indicate the use of targets for a 
+        :param semseg: boolean to indicate the use of targets for a
             semantic segmentation task. Defaults to False.
         :param labelmap_path: path (str) to a labelmap.json file,
-            that provides a dictionary mapping 'class-names'(str) to 
-            class-labels(int). The 'class-names' are derived from the 
+            that provides a dictionary mapping 'class-names'(str) to
+            class-labels(int). The 'class-names' are derived from the
             sub-directory names for each subsequence.
         """
 
         if root is None:
-            root = default_dataset_location('endless-cl-sim')
+            root = default_dataset_location("endless-cl-sim")
 
         if scenario is None and download:
             raise ValueError("No scenario defined to download!")
 
         super(EndlessCLSimDataset, self).__init__(
-            root, download=download, verbose=True)
+            root, download=download, verbose=True
+        )
 
         self.scenario = scenario
         self.patch_size = patch_size
@@ -311,20 +337,23 @@ class EndlessCLSimDataset(DownloadableDataset):
             self.patch_size = (240, 135)
 
         if self.semseg:
-            assert isinstance(self.patch_size, tuple), \
-                "If semseg is False, patch_size needs to be of type `int`"
+            assert isinstance(
+                self.patch_size, tuple
+            ), "If semseg is False, patch_size needs to be of type `int`"
         else:
-            assert isinstance(self.patch_size, int), \
-                "If semseg is True, patch_size needs to be of type `tuple`"
+            assert isinstance(
+                self.patch_size, int
+            ), "If semseg is True, patch_size needs to be of type `tuple`"
 
         # Download the dataset and initialize metadata
         self._load_dataset()
         return
 
     def _get_scenario_data(self):
-        """
-        Returns:
-            tuple: ("DataName.zip", "download-url", "MD5-checksum") of a 
+        """Get data about the scenario.
+
+        :return:
+            tuple ("DataName.zip", "download-url", "MD5-checksum") of a
             derived data to be used, as defined in endless_cl_sim_data.py
         """
         data = endless_cl_sim_data.data
@@ -347,12 +376,10 @@ class EndlessCLSimDataset(DownloadableDataset):
         raise ValueError("Provided 'scenario' parameter is not valid!")
 
     def _prepare_classification_subsequence_datasets(self, path) -> bool:
-        """
-        Args:
-            path (str): Path to the root of the data to be loaded.
+        """Prepare subsequences.
 
-        Returns:
-            success (bool): Boolean wether the preparation was successfull.
+        :param path: (str) Path to the root of the data to be loaded.
+        :return: success (bool): Boolean wether the preparation was successfull.
         """
         # Get sequence dirs
         sequence_paths = glob.glob(path + os.path.sep + "*" + os.path.sep)
@@ -360,17 +387,19 @@ class EndlessCLSimDataset(DownloadableDataset):
         # For every sequence (train, test)
         for sequence_path in sequence_paths:
             sub_sequence_paths = glob.glob(
-                sequence_path + os.path.sep + "*" + os.path.sep)
-            # Get sub-sequence dirs (0,1,....,n)       
+                sequence_path + os.path.sep + "*" + os.path.sep
+            )
+            # Get sub-sequence dirs (0,1,....,n)
             for sub_sequence_path in sub_sequence_paths:
                 image_paths = []
                 targets = []
 
                 # Get class dirs
-                class_name_dirs = [f.name for f
-                                   in os.scandir(
-                                       sub_sequence_path + os.path.sep) 
-                                   if f.is_dir()]
+                class_name_dirs = [
+                    f.name
+                    for f in os.scandir(sub_sequence_path + os.path.sep)
+                    if f.is_dir()
+                ]
 
                 # Load file_paths and targets
                 for class_name in class_name_dirs:
@@ -381,10 +410,13 @@ class EndlessCLSimDataset(DownloadableDataset):
 
                 # Create sub-sequence dataset
                 subsequence_dataset = ClassificationSubSequence(
-                        image_paths, targets, patch_size=self.patch_size,
-                        labelmap_path=self.labelmap_path,
-                        transform=self.transform, 
-                        target_transform=self.target_transform)
+                    image_paths,
+                    targets,
+                    patch_size=self.patch_size,
+                    labelmap_path=self.labelmap_path,
+                    transform=self.transform,
+                    target_transform=self.target_transform,
+                )
                 if "train" in (sequence_path.lower()):
                     self.train_sub_sequence_datasets.append(subsequence_dataset)
                 elif "test" in (sequence_path.lower()):
@@ -392,16 +424,20 @@ class EndlessCLSimDataset(DownloadableDataset):
                 else:
                     raise ValueError(
                         "Sequence path contains neighter 'train' nor \
-                            'test' identifier!")
+                            'test' identifier!"
+                    )
 
         # Check number of train and test subsequence datasets are equal
         if self.verbose:
-            print("Num train subsequences:", 
-                  len(self.train_sub_sequence_datasets),
-                  "Num test subsequences:", 
-                  len(self.test_sub_sequence_datasets))
-        assert(len(self.train_sub_sequence_datasets)
-               == len(self.test_sub_sequence_datasets))
+            print(
+                "Num train subsequences:",
+                len(self.train_sub_sequence_datasets),
+                "Num test subsequences:",
+                len(self.test_sub_sequence_datasets),
+            )
+        assert len(self.train_sub_sequence_datasets) == len(
+            self.test_sub_sequence_datasets
+        )
 
         # Has run without errors
         if self.verbose:
@@ -418,12 +454,10 @@ class EndlessCLSimDataset(DownloadableDataset):
         return sequence_indices
 
     def _prepare_video_subsequence_datasets(self, path) -> bool:
-        """
-        Args:
-            path (str): Path to the root of the data to be loaded.
+        """Prepare video subsequence datasets.
 
-        Returns:
-            success (bool): Boolean wether the preparation was successfull.
+        :param path: (str) Path to the root of the data to be loaded.
+        :return: success (bool) Boolean wether the preparation was successfull.
         """
         # Get sequence dirs
         sequence_paths = glob.glob(path + os.path.sep + "*" + os.path.sep)
@@ -445,15 +479,17 @@ class EndlessCLSimDataset(DownloadableDataset):
                     dir_name = data_content.split(os.path.sep)[-1]
                     if "Color" == dir_name:
                         # Extend color path
-                        color_path = data_content + os.path.sep + "0" +\
-                            os.path.sep
+                        color_path = (
+                            data_content + os.path.sep + "0" + os.path.sep
+                        )
                         # Get all files
                         for file_name in sorted(os.listdir(color_path)):
                             image_paths.append(color_path + file_name)
                     elif "Seg" == dir_name:
                         # Extend seg path
-                        seg_path = data_content + os.path.sep + "0" +\
-                            os.path.sep
+                        seg_path = (
+                            data_content + os.path.sep + "0" + os.path.sep
+                        )
                         # Get all files
                         for file_name in sorted(os.listdir(seg_path)):
                             target_paths.append(seg_path + file_name)
@@ -480,32 +516,35 @@ class EndlessCLSimDataset(DownloadableDataset):
                 print("All metadata checks complete!")
 
             sequence_indices = self._load_sequence_indices(
-                sequence_file=sequence_file)
+                sequence_file=sequence_file
+            )
 
             if self.verbose:
                 print("Sequence file loaded..")
 
             for i in range(len(sequence_indices)):
                 last_index = sequence_indices[i]
-                if (i+1) == len(sequence_indices):
+                if (i + 1) == len(sequence_indices):
                     next_index = len(image_paths)
                 else:
-                    next_index = sequence_indices[i+1]
+                    next_index = sequence_indices[i + 1]
 
                 image_subsequence_paths = image_paths[last_index:next_index]
                 target_subsequence_paths = target_paths[last_index:next_index]
 
-                assert(len(image_subsequence_paths) ==
-                       len(target_subsequence_paths))
+                assert len(image_subsequence_paths) == len(
+                    target_subsequence_paths
+                )
 
                 # Create subsequence dataset
-                subsequence_dataset = \
-                    VideoSubSequence(image_subsequence_paths, 
-                                     target_subsequence_paths,
-                                     segmentation_file, 
-                                     patch_size=self.patch_size,
-                                     transform=self.transform,
-                                     target_transform=self.target_transform)
+                subsequence_dataset = VideoSubSequence(
+                    image_subsequence_paths,
+                    target_subsequence_paths,
+                    segmentation_file,
+                    patch_size=self.patch_size,
+                    transform=self.transform,
+                    target_transform=self.target_transform,
+                )
                 if "train" in (sequence_path.lower()):
                     self.train_sub_sequence_datasets.append(subsequence_dataset)
                 elif "test" in (sequence_path.lower()):
@@ -513,20 +552,21 @@ class EndlessCLSimDataset(DownloadableDataset):
                 else:
                     raise ValueError(
                         "Sequence path contains neighter 'train' nor \
-                             'test' identifiers!")
+                             'test' identifiers!"
+                    )
         return True
 
     def __getitem__(self, index):
-        """
-        Args:
-            index (int): Index
+        """Index dataset.
 
-        Returns:
-            tuple: (TrainSubSeqquenceDataset, TestSubSequenceDataset), 
+        :param index: Index
+        :return: tuple (TrainSubSeqquenceDataset, TestSubSequenceDataset),
             the i-th subsequence data, as requested by the provided index.
         """
-        return self.train_sub_sequence_datasets[index], \
-            self.test_sub_sequence_datasets[index]
+        return (
+            self.train_sub_sequence_datasets[index],
+            self.test_sub_sequence_datasets[index],
+        )
 
     def __len__(self):
         return len(self.train_sub_sequence_datasets)
@@ -537,9 +577,9 @@ class EndlessCLSimDataset(DownloadableDataset):
         if self.verbose:
             print("Downloading " + data_name[1] + "...")
         file = self._download_file(data_name[1], data_name[0], data_name[2])
-        if data_name[1].endswith('.zip'):
+        if data_name[1].endswith(".zip"):
             if self.verbose:
-                print(f'Extracting {data_name[0]}...')
+                print(f"Extracting {data_name[0]}...")
             extract_subdir = data_name[0].split(".")[0]
             extract_root = self._extract_archive(file, extract_subdir)
 
@@ -547,12 +587,14 @@ class EndlessCLSimDataset(DownloadableDataset):
             extract_root_file_list = glob.glob(str(extract_root) + "/*")
             for file_name in extract_root_file_list:
                 sub_file_name = file_name.split("/")[-1]
-                extract_subsubdir = extract_subdir + \
-                    "/" + sub_file_name.split(".")[0]
+                extract_subsubdir = (
+                    extract_subdir + "/" + sub_file_name.split(".")[0]
+                )
                 if self.verbose:
                     print(f"Extracting: {sub_file_name} to {extract_subdir}")
                 self._extract_archive(
-                    file_name, extract_subdir, remove_archive=True)
+                    file_name, extract_subdir, remove_archive=True
+                )
                 if self.verbose:
                     print("Extraction complete!")
 
@@ -575,35 +617,41 @@ class EndlessCLSimDataset(DownloadableDataset):
                     if (self.root / name).exists():
                         if match_path is not None:
                             raise ValueError(
-                                "Two directories match the selected scenario!")
+                                "Two directories match the selected scenario!"
+                            )
                         match_path = str(self.root / name)
 
             if match_path is None:
                 return False
 
-            if not self.semseg:     
-                is_subsequence_preparation_done = \
+            if not self.semseg:
+                is_subsequence_preparation_done = (
                     self._prepare_classification_subsequence_datasets(
-                        match_path)
+                        match_path
+                    )
+                )
             else:
-                is_subsequence_preparation_done = \
+                is_subsequence_preparation_done = (
                     self._prepare_video_subsequence_datasets(match_path)
+                )
 
             if is_subsequence_preparation_done and self.verbose:
                 print("Data is loaded..")
             else:
-                return False    
+                return False
             return True
 
         # If a 'generic'-endless-cl-sim-scenario has been selected
         if not self.semseg:
-            is_subsequence_preparation_done = \
+            is_subsequence_preparation_done = (
                 self._prepare_classification_subsequence_datasets(
-                    str(self.root))
+                    str(self.root)
+                )
+            )
         else:
-            is_subsequence_preparation_done = \
-                self._prepare_video_subsequence_datasets(
-                    str(self.root))
+            is_subsequence_preparation_done = (
+                self._prepare_video_subsequence_datasets(str(self.root))
+            )
 
         if is_subsequence_preparation_done and self.verbose:
             print("Data is loaded...")
@@ -615,19 +663,18 @@ class EndlessCLSimDataset(DownloadableDataset):
 
     def _download_error_message(self) -> str:
         scenario_data_name = self._get_scenario_data()
-        all_urls = [
-            name_url[1] for name_url in scenario_data_name
-        ]
+        all_urls = [name_url[1] for name_url in scenario_data_name]
 
-        base_msg = \
-            '[Endless-CL-Sim] Error downloading the dataset!\n' \
-            'You should download data manually using the following links:\n'
+        base_msg = (
+            "[Endless-CL-Sim] Error downloading the dataset!\n"
+            "You should download data manually using the following links:\n"
+        )
 
         for url in all_urls:
-            base_msg += url 
-            base_msg += '\n'
+            base_msg += url
+            base_msg += "\n"
 
-        base_msg += 'and place these files in ' + str(self.root)
+        base_msg += "and place these files in " + str(self.root)
 
         return base_msg
 
@@ -638,14 +685,17 @@ if __name__ == "__main__":
     from torchvision import transforms
     import torch
 
-    _default_transform = transforms.Compose([
-        transforms.ToTensor()])
+    _default_transform = transforms.Compose([transforms.ToTensor()])
 
     # data = EndlessCLSimDataset(scenario="Classes", root="/data/avalanche",
     #                           semseg=True, transform=_default_transform)
-    data = EndlessCLSimDataset(scenario=None, download=False, 
-                               root="/data/avalanche/IncrementalClasses_Video",
-                               semseg=True, transform=_default_transform)
+    data = EndlessCLSimDataset(
+        scenario=None,
+        download=False,
+        root="/data/avalanche/IncrementalClasses_Video",
+        semseg=True,
+        transform=_default_transform,
+    )
 
     print("num subsequence:", len(data.train_sub_sequence_datasets))
 
@@ -654,7 +704,8 @@ if __name__ == "__main__":
 
     print(
         f"num samples in subsequence {sub_sequence_index} \
-            = {len(subsequence)}")
+            = {len(subsequence)}"
+    )
 
     dataloader = DataLoader(subsequence, batch_size=1)
 
@@ -662,7 +713,7 @@ if __name__ == "__main__":
         print(i)
         print(img.shape)
         img = torch.squeeze(img)
-        img = transforms.ToPILImage()(img)        
+        img = transforms.ToPILImage()(img)
         print("img size:", img.size)
         print("targets:", np.unique(target))
         # plt.imshow(img)
@@ -671,6 +722,4 @@ if __name__ == "__main__":
     print("Done...")
 
 
-__all__ = [
-    'EndlessCLSimDataset'
-]
+__all__ = ["EndlessCLSimDataset"]
