@@ -6,10 +6,16 @@ from .gdumb import GDumbPlugin
 from .gem import GEMPlugin
 from .lwf import LwFPlugin
 from .replay import ReplayPlugin
-from .strategy_plugin import StrategyPlugin
+from .strategy_plugin import SupervisedPlugin
 from .synaptic_intelligence import SynapticIntelligencePlugin
 from .gss_greedy import GSS_greedyPlugin
 from .cope import CoPEPlugin, PPPloss
 from .lfl import LFLPlugin
 from .early_stopping import EarlyStoppingPlugin
 from .lr_scheduling import LRSchedulerPlugin
+from .generative_replay import (
+    GenerativeReplayPlugin,
+    TrainGeneratorAfterExpPlugin,
+)
+from .rwalk import RWalkPlugin
+from .mas import MASPlugin
